
from bvhparser import *
from consts import *
from bvhwriter import *

class AnimatedFigure(object):
    def __init__(self, file_name, root, playbackinfo):
        
        self.file_name = file_name
        self.root = root
        self.joint_list = self.root.get_joint_list()
       
        self.current_joint = 0
        
        for child in self.CurrentJoint.children:
            child.current = True
            
        self.joint_highlighting = True
        
        self.playbackinfo = playbackinfo
        
        

    def deactivate_current_joint_highlighting(self):
        self.joint_highlighting = False
        
        for child in self.CurrentJoint.children:
            child.current = False
            
        



    def get_file_name(self):
        return self.file_name
    
    FileName = property(get_file_name)


    
    def display(self, current_frame):
        self.root.display(current_frame)
        
        
    def get_lookat_point(self, current_frame):
        return self.root.get_lookat_point(current_frame)
    
    
    def get_lookat_height(self):
        return self.root.get_lookat_height()
        
  
    LookAtHeight = property(get_lookat_height)
  
    def next_joint(self):
        
        if self.joint_highlighting:
        
            for child in self.CurrentJoint.children:
                child.current = False
        
            self.current_joint += 1
        
            if self.current_joint >= len(self.joint_list):
                self.current_joint = 0
            
            for child in self.CurrentJoint.children:
                child.current = True
        
        
    def previous_joint(self):
        
        if self.joint_highlighting:
        
            for child in self.CurrentJoint.children:
                child.current = False
        
            self.current_joint -= 1
        
            if self.current_joint < 0:
                self.current_joint = len(self.joint_list) - 1
            
            for child in self.CurrentJoint.children:
                child.current = True
        
        
    def get_current_joint(self):
        return self.joint_list[self.current_joint]
    
    CurrentJoint = property(get_current_joint)
    
    
    def get_animation_speed(self):
        
        return int(self.playbackinfo.FrameRateInMilliseconds)
    
    
    FrameRateInMillseconds = property(get_animation_speed)
    
    
    def get_frame_count(self):
        return self.playbackinfo.FrameCount
    
    FrameCount = property(get_frame_count)
    
    
    def remove_redundant_point(self):
        
        candidates_list = []
        
        for joint in self.joint_list:
            
            candidate = joint.get_redundant_point_tuple()
            
            if candidate != None:
            
                candidates_list.append(candidate)
               
                if abs(candidate[0]) < 0.00001:
                    
                    break
                
                
        candidates_list.sort()
        
        #tuple format (distance, joint)
        
        
        if len(candidates_list) > 0:
               
            joint = candidates_list[0][1]
            
           
            joint.remove_chosen_point()
            
            
            
    def write_to_file(self, oldfilename, newfilename):
        
        
        
        motion_lines_by_frame = self.get_points_dump()
       
        
        bvhwriter = BvhWriter()
        bvhwriter.write_new_motion_file(oldfilename, newfilename, motion_lines_by_frame)
        
        
        
        
        
    def get_points_dump(self):
        
        
        motions = []
        
        for j in self.joint_list:
            motions += j.get_motion_data_lists()
            
            
        
        channels_list = []
            
        for i in range(self.FrameCount):
            
            f = []
            
            for m in motions:
                f.append(m[i])
                
            channels_list.append(f)
            
        return channels_list
            
    
    
    def get_compression_string(self):
        
        points = 0.0
        total_points = 0.0
        
        for joint in self.joint_list:
            points += joint.PointsRemaining
            total_points += joint.TotalRemovablePoints            
        return str(points)
            
    
    CompressionString = property(get_compression_string)
    
    

    
    def get_compression_string_pretty(self):
        
        points = 0.0
        total_points = 0.0
        
        for joint in self.joint_list:
            points += joint.PointsRemaining
            total_points += joint.TotalRemovablePoints
            
        return str(int(points)) + " of " + str(int(total_points)) + " (" + str(points/total_points*100) + " %)"
            
    
    CompressionStringPretty = property(get_compression_string_pretty)
    
    def get_existing_removable_points(self):
        
        points = 0
        
        for joint in self.joint_list:
            points += joint.PointsRemaining
    
        return points
    
    
    RemainingRemovablePoints = property(get_existing_removable_points)
    
    def get_total_removable_points(self):
        
        points = 0
        
        for joint in self.joint_list:
            points += joint.TotalRemovablePoints
    
        return points
    
    
    
    
    TotalRemovablePoints = property(get_total_removable_points)
    
    
    
    
    