from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *



from graph import *








#abstract class
class Joint(object):
    def __init__(self, name, offset):
        self.name = name
        self.parent = None
        self.Offsets = offset
        self.current = False
        
    def get_name(self):
        return self.name
        
    Name = property(get_name)
        
    def get_offsets(self):
        
        return self.offsetlist
    
    def set_offsets(self, offsetlist):
        self.offsetlist = offsetlist
        if len(offsetlist) != 3:
            raise ValueError, "Bad Offsets"
        
        
    Offsets = property(get_offsets, set_offsets)
        
    
    def get_parent(self):
        return self.parent
    
    def set_parent(self, parent):
        self.parent = parent
        
    Parent = property(get_parent, set_parent)
    
    
    
    
        
    def print_recursive(self, indentation):
        for i in range(indentation):
            print ' ',
        print self.name
        
    #abstract method
    def display(self, framenumber):
        
        self.predisplay(framenumber)
        
        if self.current:
            glColor3f(0,1,1)
        else:
            glColor3f(1,1,1)
        
        glBegin(GL_LINES);
        glVertex3f (0.0, 0.0, 0.0);
        glVertex3f (self.Offsets[0], self.Offsets[1], self.Offsets[2])
        glEnd();
        
        self.postdisplay(framenumber)
    
    
    
    def predisplay(self, framenumber):
        pass

    def postdisplay(self, framenumber):
        pass
    
    
    def add_motion_lists(self, motion):
        pass
    
    
    def get_sibling_number(self):
        return self.sibling_number
    
    
    def set_sibling_number(self, num):
        self.sibling_number = num
        
        
    SiblingNumber = property(get_sibling_number, set_sibling_number)
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
        
        
class InternalJoint(Joint):
    def __init__(self, name, offset):
        Joint.__init__(self, name, offset)
        
        self.children = []
        self.channels = []
        self.graphs = []
        
        


    def postdisplay(self, framenumber):

    
        glPushMatrix()
    
        self.display_children(framenumber)
        
        glPopMatrix()
        
        
        
        
    def display_children(self, framenumber):
        glTranslatef(self.Offsets[0], self.Offsets[1], self.Offsets[2])
    
        glRotatef(self.graphs[0].DisplayValues(framenumber), 0, 0, 1)
        glRotatef(self.graphs[1].DisplayValues(framenumber), 1, 0, 0)
        glRotatef(self.graphs[2].DisplayValues(framenumber), 0, 1, 0)
    
        
        for child in self.children:
            child.display(framenumber)


    def get_motion_data_lists(self):
        l = []
        
        for g in self.graphs:
            l.append(g.get_current_channel_data_copy())
            
        return l


    def add_child(self, child):
        child.SiblingNumber = len(self.children)
        self.children.append(child)
        child.Parent = self
        
        
    def graphs_init(self):
        
        for channel in self.channels:
            
            graph = Graph(channel, self)
            self.graphs.append(graph)
        
        
    def print_recursive(self, indentation):
        
        
        for i in range(indentation):
            print ' ',
            
        if self.Parent != None:
            print self.name, "son of", self.Parent.Name, len(self.channels)
        else:
            print self.name, len(self.channels)
        
        print self.channels[0][0:3]

        for child in self.children:
            child.print_recursive(indentation+2)


    def add_motion_lists(self, motion):
        
        for i in range(3):
            self.channels.append(motion.get_next_channel())
            
        self.graphs_init()
            
        for child in self.children:
            child.add_motion_lists(motion)
            
            
            
        
    def get_joint_list(self):
        list = [self]
        
        for child in self.children:
            if (isinstance(child, InternalJoint)):
                list += child.get_joint_list()
                
        return list
        
            
    def get_angular_error(self):
        
        total = 0
        
        for g in self.graphs:
            total += g.get_angular_error()
            
        return total

    AngularError = property(get_angular_error)





    def get_points_remaining(self):
        
        points = 0
        
        for g in self.graphs:
            points += g.PointsRemaining
        
        return points
        
    PointsRemaining = property(get_points_remaining)
    
    def get_total_points(self):
        return self.graphs[0].TotalPoints * 3
    
    TotalPoints = property(get_total_points)


    def get_total_removable_points(self):
        return (self.graphs[0].TotalPoints - EXEMPT_POINTS) * len(self.graphs)
    
    TotalRemovablePoints = property(get_total_removable_points)



    def get_total_endpoint_diff(self, graph, frame_numbers, interpolated_values):
        
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        
        
        totaldiff = 0.0
        
        for index, framenumber in enumerate(frame_numbers):
            
            
            
            rotations = []
            
            identical = False
            for g in self.graphs:
                if g == graph:
                    if interpolated_values[index] == g.DisplayValues(framenumber):
                        identical = True
                        break
                    
                    
                    rotations.append(interpolated_values[index])
                else:
                    rotations.append(g.DisplayValues(framenumber))
                    
                    
            if identical:
                continue
                    
                    
            glPushMatrix()
            
            glTranslatef(self.Offsets[0], self.Offsets[1], self.Offsets[2])
            glRotatef(self.graphs[0].DisplayValues(framenumber), 0, 0, 1)
            glRotatef(self.graphs[1].DisplayValues(framenumber), 1, 0, 0)
            glRotatef(self.graphs[2].DisplayValues(framenumber), 0, 1, 0)
        
            original_endpoints = []
        
            self.recursive_only_get_endpoint_position(original_endpoints, framenumber)
            
        
            glPopMatrix()
            
            
                
            glPushMatrix()
        
            
            glTranslatef(self.Offsets[0], self.Offsets[1], self.Offsets[2])
            glRotatef(rotations[0], 0, 0, 1)
            glRotatef(rotations[1], 1, 0, 0)
            glRotatef(rotations[2], 0, 1, 0)
            
            
            modified_endpoints = []
        
            self.recursive_only_get_endpoint_position(modified_endpoints, framenumber)
        
            glPopMatrix()
        
            for i, endpoint in enumerate(original_endpoints): 
                diff0 = endpoint[0] - modified_endpoints[i][0]
                diff1 = endpoint[1] - modified_endpoints[i][1]
                diff2 = endpoint[2] - modified_endpoints[i][2]
                
                totaldiff += pyth(diff0, diff1, diff2)
            
        return totaldiff
                
                


    def recursive_only_get_endpoint_position(self, list, framenumber):
        
        for child in self.children:
            glPushMatrix()
            
            glTranslatef(self.Offsets[0], self.Offsets[1], self.Offsets[2])
            glRotatef(self.graphs[0].DisplayValues(framenumber), 0, 0, 1)
            glRotatef(self.graphs[1].DisplayValues(framenumber), 1, 0, 0)
            glRotatef(self.graphs[2].DisplayValues(framenumber), 0, 1, 0)
            
            child.recursive_only_get_endpoint_position(list, framenumber)
            glPopMatrix()
        
        
        
        
        




class LeafJoint(Joint):
    def __init__(self, name, offset):
        Joint.__init__(self, name, offset)
        
    def display(self, framenumber):
        import ctypes
        
        
        if self.current:
            glColor3f(0,1,1)
        else:
            glColor3f(1,1,1)
        
        glBegin(GL_LINES);
        glVertex3f (0.0, 0.0, 0.0);
        glVertex3f (self.Offsets[0], self.Offsets[1], self.Offsets[2])
        glEnd();
    
        
    
    def recursive_only_get_endpoint_position(self, list, framenumber):
        glTranslatef(self.Offsets[0], self.Offsets[1], self.Offsets[2])
        modelview_matrix = glGetFloatv(GL_MODELVIEW_MATRIX)
        
        point = [modelview_matrix[3][0], modelview_matrix[3][1], modelview_matrix[3][2]]
                 
        list.append(point)
        
        
        
        
        
        
        
class RootJoint(InternalJoint):
    def __init__(self, name, offset):
        InternalJoint.__init__(self, name, offset)
        
        self.transchannels = []
        self.transgraphs = []
        
        self.SiblingNumber = 0
    
    
    
    
    
    def predisplay(self, framenumber):
        glPushMatrix()
        
        glTranslatef(self.transchannels[0][framenumber], self.transchannels[1][framenumber], self.transchannels[2][framenumber])
        glColor3f(1,1,1)
    
    
        
    def postdisplay(self, framenumber):
        
    
        self.display_children(framenumber)
            
        glPopMatrix()
        
    def get_motion_data_lists(self):
        l = []
        
        for t in self.transchannels:
            l.append(list(t))
        
        
        for g in self.graphs:
            l.append(g.get_current_channel_data_copy())
            
        return l

    def get_lookat_point(self, framenumber):
        return [self.transchannels[0][framenumber], self.transchannels[1][framenumber], self.transchannels[2][framenumber]]

    def get_lookat_height(self):
        return sum(self.transchannels[1]) / len(self.transchannels[1])

    def print_info(self):
        self.print_recursive(0)
        
        
    def integrate(self, motion):
        self.add_motion_lists(motion)
        
        
    def add_motion_lists(self, motion):
        
        for i in range(3):
            self.transchannels.append(motion.get_next_channel())
        
        
        for i in range(3):
            self.channels.append(motion.get_next_channel())
            
        self.graphs_init()
        
        for child in self.children:
            child.add_motion_lists(motion)
            
            
    
    
        
